function [ boolean ] = create_csv_table( p_metadata, p_config )
%CREATE_CSV_TABLE Creates a csv file containing table with information of all floats.
%
%   Parameters:
%   p_metadata:        Struct containing all floats (load with nemo.load.data)
%   p_config:          Struct containing global configuration file (load with
%                      nemo.load.config()).
%
%   Returns:
%   boolean:  False if something went wrong during the creation of the table.

%% Initialize return variable
boolean = false;

%% Parameter check

if (~iscell(p_metadata) || ~isstruct(p_config))
    disp([mfilename ': Wrong parameter. Please check your input!']);
    return
end

%% Collect all required information

serial = [];
imei = [];
wmoid = [];
deployment_datetime = [];
deployment_lat = [];
deployment_lon = [];
profile_count = [];
latest_transmission = [];

float_count = length(p_metadata);

for o_float = 1:float_count % collect all required information of every float
    % load the latest float
    [current_profile, current_profile_count] = nemo.load.latest_profile( ...
        p_config, ...
        p_metadata{o_float}.GENERAL_INFO.SERIAL ...
    );

    % get all numbers of nemo floats
    serial(end+1, 1) = p_metadata{o_float}.GENERAL_INFO.SERIAL;
    imei(end+1, 1) = p_metadata{o_float}.GENERAL_INFO.IMEI;
    wmoid(end+1, 1) = p_metadata{o_float}.GENERAL_INFO.WMO_ID;
    % deploment info
	deployment_datetime(end+1, :) = datevec( ...
        [ ...
            p_metadata{o_float}.DEPLOYMENT_INFO.DEPLOYMENT_DATE ...
            p_metadata{o_float}.DEPLOYMENT_INFO.DEPLOYMENT_TIME ...
        ], 'dd.mm.yyyyHH:MM');


%
%     [ ...
%         p_metadata{o_float}.DEPLOYMENT_INFO.deployment_date(3), ...
%         current_profile.DEPLOYMENT_INFO.deployment_date(2), ...
%         current_profile.DEPLOYMENT_INFO.deployment_date(1), ...
%         current_profile.DEPLOYMENT_INFO.deployment_time, ...
%         0];
    deployment_lat(end+1, 1) = p_metadata{o_float}.DEPLOYMENT_INFO.DEPLOYMENT_LAT;
    deployment_lon(end+1, 1) = p_metadata{o_float}.DEPLOYMENT_INFO.DEPLOYMENT_LON;
    % number of received profiles
	profile_count(end+1, 1) = current_profile_count;
	% last float transmittion
    if ~isempty(current_profile) && ~isempty(current_profile.IRIDIUM_DATA)
        latest_transmission(end+1, :) = current_profile.IRIDIUM_DATA(4:9);
    else
        latest_transmission(end+1, :) = nan(1, 6);
    end

end

% create filename
filename = fullfile( ...
    p_config.ANALYSIS_RESULTS.output_path, ...
    p_config.ANALYSIS_RESULTS.compact_info_filename ...
    );

% create folder if necessary
if ~exist(p_config.ANALYSIS_RESULTS.output_path, 'dir')
    mkdir(p_config.ANALYSIS_RESULTS.output_path);
end

% create header of table
table_header = { ...
    'serial', 'imei', 'wmo_id', 'deployment_time', 'deployment_lat', 'deployment_lon', ...
    'latest_transmission', 'profile_count' ...
};

% create matlab table with the collected information
compact_info = table(serial, imei, wmoid, datetime(deployment_datetime, 'Format', 'yyyy-MM-dd HH:m'), deployment_lat, deployment_lon, ...
    datetime(latest_transmission, 'Format', 'yyyy-MM-dd HH:mm'), profile_count, 'VariableNames', table_header);

% write data to file
writetable(compact_info, filename, 'Delimiter', ',');
try
% copy template file to csv folder
copyfile(p_config.ANALYSIS_RESULTS.compact_info_html_template, [ ...
    p_config.ANALYSIS_RESULTS.output_path ...
    'index.html' ...
]);
% update its timestamp via java
import java.io.File java.text.SimpleDateFormat
f = File([ p_config.ANALYSIS_RESULTS.output_path 'index.html' ]);
sdf = SimpleDateFormat('HH:mm dd/MM/yyyy');
today = datetime('now');
newDate = sdf.parse([ ...
    int2str(today.Hour) ':' int2str(today.Minute) ' ' ...
    int2str(today.Day) '/' int2str(today.Month) '/' int2str(today.Year) ...
]);
f.setLastModified(newDate.getTime);
clear f;
% copy javascript folder
copyfile(p_config.ANALYSIS_RESULTS.input_javascript_folder, ...
    [p_config.ANALYSIS_RESULTS.output_path 'js' p_config.GENERAL.folder_separator] ...
);
catch ex
warning('Could not copy templates or JavaScript folder. If you experience Problems try deleting those files/folders and rerun!');
boolean = false;
return;
end

% everything went fine! :)
boolean = true;

end
